/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/*
 *  
 * @author James Geraci
 * 
 */
#ifndef _CV_H
#define _CV_H
#include "StdAfx.h"
class FV;

class CV
{
public:
	CV(void);
public:
	~CV(void);
	
	double returnmbScale(void);
	void setpvVolume(double);

	bool exist;
	double myXnorm;
	double myYnorm;

	FV* myFV;
	double mydx;
	double mydy;
	double mydz;

	double mydx2;
	double invmydx;
	double invmydx2;
	double mydxsqrd;

	double mydy2;
	double invmydy;
	double invmydy2;
	double mydysqrd;

	double mydz2;
	double invmydz;
	double invmydz2;
	double mydzsqrd;

	void setPosition(int);
	double dhdtCV;
	double dhdt;

	double docCV;

	double ekCV;

	void computedxSize(void);
	void computedySize(void);
	void computedzSize(void);
	void computemyVolume(void);
	double returnmyVolume(void);
//  MB
	double setdhdt(void);
	double setdhdtCV(void);
	double myJhp(void);
	double myJxhp(void);
	double myJyhp(void);
	double myFdx(void);
	double myFdy(void); 

	// DOC
	double setdocCV(void);
	double setdocCVjl(void);
	double setdocCVse(void);
	double myFjlx(void);
	double myFsex(void);
	double myFjly(void); 
	double myFsey(void);
	double myJphil(void);
	double myJxphil(void);
	double myJyphil(void);
	double myDOCJhp(void);
	double myDOCJxhp(void);
	double myDOCJyhp(void);
	double myJphis(void);
	double myJxphis(void);
	double myJyphis(void);
	double inCurrentCV(void);   //  Used only with the top row
// EK
	double setekCV(void);
	double myArea(void);
	double expTerm(void);
	double ekJeps(void);
	double ekJhp(void);
	double ekJphis(void);
	double ekJphil(void);
private:
	int myPosition;
	double gls; 
	double gsl;
	double overp;
	double myexpTerm;
    double myFVdx;
	double mzFVdy;
	double myVolume;
	double pvVolume;
	double myPVvolumeFraction;
	double Area;

public:
		// Data Matrix Pointers
	double* fluxVptr;
	double* pVptr;
	double* Vptr;
	double* iVptr;
	double** Jacobptr;

	// Position information for Cell
	int myColumn;
	int myRow;
	int myZrow;
	int JRow;
	int JRowBuffer;

    // Positioning Data
	int rowLength;
	int maxPvIndex;

	// FV grid Information
	int numFVColumns;
	int numFVRows;
	
	//  Used in Calculating Addresses
	int prevV;
	int presV;
	int nextV;

	int topV;
	int lowerV;
	// present volume's properties
	int hpc;
	int phisc;
	int philc;
// previous volume's properties
	int phpc;
	int pphisc;
	int pphilc;
// next volume's properties
	int nhpc;
	int nphisc;
	int nphilc;
// top volume's properties
	int thpc;
	int tphisc;
	int tphilc;
// lower volume's properties
	int bhpc;
	int bphisc;
	int bphilc;
};

#endif
