/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* 
 *  @author James Geraci
 *  Sets up the battery Cell by creating each electrode and
 *  making each electrode away of the other electrodes.
 */
#include "StdAfx.h"
#include "Electrode.h"
#include <iostream>

Electrode::Electrode(void)
{
}

Electrode::Electrode(char* pos, FV** myFv, PV** myPv, matrixData* myMatrix, electrodeSizeData* eData,
					 electrodeChemData myChem, volumeChemData myVol, volumeIConditions iData, PVIConditions pviData)
{
	myData = myMatrix;
	myeData = eData;
	myelectrodeChemData = myChem;
	myvCData = myVol;
	myiData = iData;
	mypviData = pviData;


	mycsData = eData->mycsData;
	numFVColumns = eData->getnumFVColumns();
	numFVRows = eData->getnumFVRows();
	numFVZrows = eData->getnumFVZrows();

	numFVVolumes = numFVColumns*numFVRows*numFVZrows;

	numPVColumns = eData->getnumPVColumns();
	numPVRows = eData->getnumPVRows();
	numPVZrows = eData->getnumPVZrows();

	numPVVolumes = numPVColumns*numPVRows*numPVZrows;


	yHeight = eData->getyHeight();
	xWidth = eData->getxWidth();
	zDepth = eData->getzDepth();

	FVvdx = (double) xWidth/numFVColumns;
	FVvdy = (double) yHeight/numFVRows;
	FVvdz = (double) zDepth/numFVZrows;

	
	PVvdx = (double) xWidth/numFVColumns;
	PVvdy = (double) yHeight/numFVRows;
	PVvdz = (double) zDepth/numFVZrows;

	FVstartColumn = myeData->getFVstartColumn();
	PVstartColumn = myeData->getPVstartColumn();

	try{
		vsData = (volumeSpatialData**) new char [numFVVolumes*sizeof(volumeSpatialData*)]; 
		}
	catch(std::bad_alloc){
	  std::cout << "Couldn't Allocate vsData array!" << std::endl;
		exit(-1);
		}

	for(int counter = 0; counter < numFVVolumes; counter++)
	{
		vsData[counter] = NULL;
	};

	index =0;
	presFVColumn = 0;
	presFVRow = 0;
	while(presFVRow < numFVRows)
	{
		while(presFVColumn < numFVColumns)
		{
			try{
				vsData[index] =  new volumeSpatialData(FVvdx,FVvdy,FVvdz,presFVColumn,presFVRow,0);
				}
				catch(std::bad_alloc)
				{
				std::cout << "Failed to allocate volumeSpatialData Object number " << index << std::endl;
				exit(-1);
				}
			presFVColumn++;
			index++;
			}
		presFVColumn = 0;
		presFVRow++;
		}
	

	if(!strcmp(pos,"ldx"))
	{
		makeLeftElectrode(myFv,myPv);
	}
	else if(!strcmp(pos,"Electrolyte"))
	{
		makeElectrolyte(myFv,myPv);
	}
	else if(!strcmp(pos,"ld"))
	{
		makeRightElectrode(myFv,myPv);
	}
}

Electrode::~Electrode(void)
{
	for(int counter = 0; counter < numFVVolumes; counter++)
	{
	delete vsData[counter];
	}
	for(int counter = 0; counter < numPVVolumes; counter++)
	{
	delete  PVvsData[counter];
	}
	delete [] vsData;
	delete [] PVvsData;
}


void Electrode::makeLeftElectrode(FV** myFv,PV** myPv){
		index = FVstartColumn;
	presFVRow = 0;
	presFVColumn = FVstartColumn;

		while(presFVRow < numFVRows)
		{
			while(presFVColumn < numFVColumns+FVstartColumn)
			{
			try{
				int f = index-(presFVRow*(mycsData->getElectrolyteFVColumns() 
					+ mycsData->getldFVColumns())+ FVstartColumn);
			myFv[index] =  new FV(vsData[f], myData, myeData, &myvCData, &myelectrodeChemData, &myiData);
			f = f;
			}
			catch(std::bad_alloc)
			{
			std::cout << "Cannot allocate FV number " << index << std::endl;
			exit(-1);
			}
			presFVColumn++;
			index++;
			}
			presFVColumn = FVstartColumn;
			presFVRow++;
			index = index + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns();
		}

	int aa;
	int baseIndex;
	try{
		PVvsData = (volumeSpatialData**) new char [numPVVolumes*sizeof(volumeSpatialData*)]; 
		}
	catch(std::bad_alloc){
		std::cout << "Couldn't Allocate vsData array!" << std::endl;
		exit(-1);
		}
		
	index = PVstartColumn;
	presPVRow = 0;
	presPVColumn = PVstartColumn;
	while(presPVRow < numPVRows){
		while(presPVColumn < numPVColumns+PVstartColumn){
		try{
		PVvsData[index] =  new volumeSpatialData(PVvdx,PVvdy,PVvdz,presPVColumn,presPVRow,0);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Failed to allocate volumeSpatialData Object number " << index << std::endl;
		exit(-1);
		}
		presPVColumn++;
		index++;
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	}

	//  Now initialize the Property Volumes
	//  Make the top Row
	index = PVstartColumn;
	presPVRow = 0;
	presPVColumn = PVstartColumn;
	baseIndex = numPVColumns+index-1;
	try{
	aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);

	myPv[index] =  new OLULPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);

	}
	catch(std::bad_alloc)
	{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
	}
 
	index++;
	// Inner volumes
	while(index < baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
		myPv[index] =  new UPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);

		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	// Upper Right Most volume
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
	myPv[index] =  new UPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;

/**************************************************************************************************/
/**************************  Now make Middle Rows of PV Volumes ***********************************/
/**************************************************************************************************/
	while(presPVRow != numPVRows -1){
		baseIndex = numPVColumns+index-1;
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
	myPv[index] =  new OLPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
	}
	catch(std::bad_alloc)
	{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
	}
	
	index++;
	
	// Inner volumes
	while(index < baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
		myPv[index] =  new PV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	//  Right Most volume
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
		myPv[index] =  new PV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
	}


/**************************************************************************************************/
/**************************  Now make Bottom Row of PV Volumes ************************************/
/**************************************************************************************************/
	baseIndex = numPVColumns+index-1;
	presPVColumn = PVstartColumn;
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
	myPv[index] =  new OLLLPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
	}
	catch(std::bad_alloc)
	{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
	}
	
	index++;
	// Lower volumes
	while(index < baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
		myPv[index] =  new BPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	// Lower Right Most volume
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
	       
	myPv[index] =  new BPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
	return;
}

void Electrode::makeRightElectrode(FV** myFv,PV** myPv){
		index = FVstartColumn;
	presFVRow = 0;
	presFVColumn = FVstartColumn;

		while(presFVRow < numFVRows)
		{
			while(presFVColumn < numFVColumns+FVstartColumn)
			{
			try{
				int f = index-(presFVRow*(mycsData->getElectrolyteFVColumns() 
					+ mycsData->getldxFVColumns())+ FVstartColumn);
			myFv[index] =  new FV(vsData[f], myData, myeData, &myvCData, &myelectrodeChemData, &myiData);
			f = f;
			}
			catch(std::bad_alloc)
			{
			std::cout << "Cannot allocate FV number " << index << std::endl;
			exit(-1);
			}
			presFVColumn++;
			index++;
			}
			presFVColumn = FVstartColumn;
			presFVRow++;
			index = index + mycsData->getElectrolyteFVColumns() + mycsData->getldxFVColumns();
		}

	int aa;
	int baseIndex;
	try{
		PVvsData = (volumeSpatialData**) new char [numPVVolumes*sizeof(volumeSpatialData*)]; 
		}
	catch(std::bad_alloc){
		std::cout << "Couldn't Allocate vsData array!" << std::endl;
		exit(-1);
		}
	
	index = 0;
	presPVRow = 0;
	presPVColumn = PVstartColumn;
	while(presPVRow < numPVRows){
		while(presPVColumn < numPVColumns+PVstartColumn){
		try{
		PVvsData[index] =  new volumeSpatialData(PVvdx,PVvdy,PVvdz,presPVColumn,presPVRow,0);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Failed to allocate volumeSpatialData Object number " << index << std::endl;
		exit(-1);
		}
		presPVColumn++;
		index++;
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	}
	//  Now initialize the Property Volumes
	//  Make the top Row
	index = PVstartColumn;
	presPVRow = 0;
	presPVColumn = PVstartColumn;
	baseIndex = numPVColumns+index-1;
	try{
	aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
	myPv[index] =  new UPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
	}
	catch(std::bad_alloc)
	{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
	}
	
	index++;
	// Inner volumes
	while(index < baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
		myPv[index] =  new UPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	// Upper Right Most volume
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
		myPv[index] =  new ORURPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
/**************************************************************************************************/
/**************************  Now make Middle Rows of PV Volumes ***********************************/
/**************************************************************************************************/
	while(presPVRow != numPVRows -1){
		baseIndex = numPVColumns+index-1;
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
	myPv[index] =  new PV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
	}
	catch(std::bad_alloc)
	{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
	}
	
	index++;
       
	// Inner volumes
	while(index < baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
		myPv[index] =  new PV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	//  Right Most volume
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
		myPv[index] =  new ORPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
	}


/**************************************************************************************************/
/**************************  Now make Bottom Row of PV Volumes ************************************/
/**************************************************************************************************/
baseIndex = numPVColumns+index-1;
	presPVColumn = PVstartColumn;
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
		
	myPv[index] =  new BPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
	}
	catch(std::bad_alloc)
	{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
	}
	
	index++;
	// Lower volumes
	while(index < baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
		myPv[index] =  new BPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	// Lower Right Most volume
	try{
		aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldxPVColumns())+ PVstartColumn);
		myPv[index] =  new ORLRPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
	return;
}


void Electrode::makeElectrolyte(FV** myFv,PV** myPv){
		index = FVstartColumn;
	presFVRow = 0;
	presFVColumn = FVstartColumn;

		while(presFVRow < numFVRows)
		{
			while(presFVColumn < numFVColumns+FVstartColumn)
			{
			try{
				int f = index-(presFVRow*(mycsData->getldxFVColumns() 
					+ mycsData->getldFVColumns())+ FVstartColumn);
			myFv[index] =  new FV(vsData[f], myData, myeData, &myvCData, &myelectrodeChemData, &myiData);
			f = f;
			}
			catch(std::bad_alloc)
			{
			std::cout << "Cannot allocate FV number " << index << std::endl;
			exit(-1);
			}
			presFVColumn++;
			index++;
			}
			presFVColumn = FVstartColumn;
			presFVRow++;
			index = index + mycsData->getldxFVColumns() + mycsData->getldFVColumns();
		}

	int aa;
	int baseIndex;
	try{
		PVvsData = (volumeSpatialData**) new char [numPVVolumes*sizeof(volumeSpatialData*)]; 
		}
	catch(std::bad_alloc){
		std::cout << "Couldn't Allocate vsData array!" << std::endl;
		exit(-1);
		}
	
	index = 0;
	presPVRow = 0;
	presPVColumn = PVstartColumn;
	while(presPVRow < numPVRows){
		while(presPVColumn < numPVColumns+PVstartColumn){
		try{
		PVvsData[index] =  new volumeSpatialData(PVvdx,PVvdy,PVvdz,presPVColumn,presPVRow,0);
		}
	catch(std::bad_alloc)
		{
		std::cout << "Failed to allocate volumeSpatialData Object number " << index << std::endl;
		exit(-1);
		}
		presPVColumn++;
		index++;
		}
	presPVRow++;
	presPVColumn = PVstartColumn;
	}
	//  Now initialize the Property Volumes
	//  Make the top Row
	index = PVstartColumn;
	presPVRow = 0;
	presPVColumn = PVstartColumn;
	baseIndex = numPVColumns+index-1;

	// Inner volumes
	while(index <= baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getElectrolytePVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
			myPv[index] =  new UPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
/**************************************************************************************************/
/**************************  Now make Middle Rows of PV Volumes ***********************************/
/**************************************************************************************************/
	while(presPVRow != numPVRows -1){
		baseIndex = numPVColumns+index-1;
	// Inner Volumes
	while(index <= baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getldxPVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
			myPv[index] =  new PV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}

	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
	}


/**************************************************************************************************/
/**************************  Now make Bottom Row of PV Volumes ************************************/
/**************************************************************************************************/
baseIndex = numPVColumns+index-1;
	presPVColumn = PVstartColumn;
	while(index <= baseIndex)
	{
		try{
			aa = index-(presPVRow*(mycsData->getldxPVColumns() 
					+ mycsData->getldPVColumns())+ PVstartColumn);
			myPv[index] =  new BPV(PVvsData[aa],myData,&myvCData,myeData,&mypviData);
		}
		catch(std::bad_alloc)
		{
		std::cout << "Out of Memory!" << std::endl;
		exit(-1);
		}
		index++;
	}
	presPVRow++;
	presPVColumn = PVstartColumn;
	index = presPVRow*(mycsData->getldxPVColumns() + mycsData->getElectrolyteFVColumns() + mycsData->getldFVColumns())
			+ PVstartColumn;
	return;
}

FV* Electrode::returnLeftFVptr(void){
	return myFv[0];
}

FV* Electrode::returnRightFVptr(void){
	return myFv[numFVColumns-1];
}
