/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* @author James Geraci
 */
#ifndef _ELECTRODE_H
#define _ELECTRODE_H
#include "matrixData.h"
#include "FV.h"
#include "PV.h"
#include "cellSizeData.h"

class Electrode
{
public:
	Electrode(void);
	Electrode(char*,FV** myFV, PV** myPV,matrixData*,electrodeSizeData*,electrodeChemData,volumeChemData,volumeIConditions,PVIConditions);
public:
	virtual ~Electrode(void);

	FV* returnLeftFVptr(void);
	FV* returnRightFVptr(void);

protected:
	cellSizeData* mycsData;
	int index;

	double myDx;
	double myeps;
	double mydx;
	int numFVColumns;
	int numFVRows;
	int numFVZrows;

	int numPVColumns;
	int numPVRows;
	int numPVZrows;

	double xWidth;
	double yHeight;
	double zDepth;

	double FVvdx;
	double FVvdy;
	double FVvdz;
	
	double PVvdx;
	double PVvdy;
	double PVvdz;

	int presFVColumn;
	int presFVRow;

	int presPVColumn;
	int presPVRow;

	int numFVVolumes;
	int numPVVolumes;

	int FVstartColumn;
	int PVstartColumn;

	matrixData* myData;
	volumeSpatialData** vsData;
	volumeSpatialData** PVvsData;
	electrodeSizeData* myeData;
	electrodeChemData myelectrodeChemData;
	volumeChemData myvCData;
	volumeIConditions myiData;
	PVIConditions mypviData;
	FV** myFv;
	PV** myPv;

private:
	void makeLeftElectrode(FV**,PV**);
	void makeRightElectrode(FV**,PV**);
	void makeElectrolyte(FV**,PV**);
};

#endif
