/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* 
 * @author James Geraci
 * creates a PV volume.  These store the states: concentration, phil, phis
 * and each PV contains 4 CVs
 */
#include "StdAfx.h"
#include "PV.h"
#include <iostream>
using namespace std;
PV::PV(void)
{
}

PV::PV(volumeSpatialData* sData, matrixData* mD, volumeChemData* vcData, electrodeSizeData* esData, PVIConditions* pviC){
	myCV[UL].exist = true;
	myCV[UR].exist = true;
	myCV[LL].exist = true;
	myCV[LR].exist = true;
	for(int x = 0; x < 4; x++){
		myCV[x].setPosition(x);
		if(x == UL){
			myCV[x].myXnorm = -1;
			myCV[x].myYnorm = 1;
		}
		else if(x == UR){
			myCV[x].myXnorm = 1;
			myCV[x].myYnorm = 1;
		}
		else if(x == LL){
			myCV[x].myXnorm = -1;
			myCV[x].myYnorm = -1;
		}
		else if(x == LR){
			myCV[x].myXnorm = 1;
			myCV[x].myYnorm = -1;
		}
	}

	initializeMatrix(mD);
	initializePosition(sData,mD,esData);
	initializeOtherPositions();
//	initializeSize(sData);
	initializeVolumeChem(vcData);
setInitialConditions(pviC);		
/*mD->writeVtoDisk(mD->mysV(),mD->mysvLength());
 cout << endl << mD->mysvLength();
 int myBaby;
 cin>>myBaby;*/
	return;
}

PV::~PV(void)
{
}
			
void PV::initializeMatrix(matrixData *mData)
{
	fluxVptr = mData->myfluxV();
	divVptr = mData->mydivV();
	pVptr = mData->mypsV();
	Vptr = mData->mysV();
	iVptr = mData->myiV();
	Jacobptr = mData->myJacobian(); 
	numFVColumns = mData->myFVColumns();
	numFVRows = mData->myFVRows();
	
	for(int x = 0; x < 4; x++){
	myCV[x].fluxVptr = mData->myfluxV();
	myCV[x].pVptr = mData->mypsV();
	myCV[x].Vptr = mData->mysV();
	myCV[x].iVptr = mData->myiV();
	myCV[x].Jacobptr = mData->myJacobian(); 
	myCV[x].numFVColumns = mData->myFVColumns();
	myCV[x].numFVRows = mData->myFVRows();
	}
	return;
}

void PV::initializePosition(volumeSpatialData *sData, matrixData* mData, electrodeSizeData* eData)
{
	myColumn = sData->getColumn();
	myRow = sData->getRow();
	myZrow = sData->getZrow();
	rowLength = mData->mysvLength();
	rowLength = mData->mysvLength();
	numPVvolumes = mData->myPVColumns()*mData->myPVRows()*mData->myPVZrows();
	maxPvIndex = mData->myPVColumns()*mData->myPVRows()*mData->myPVZrows()-1;

	presV = /*eData->getPVstartColumn()+*/myColumn + myRow*mData->myPVColumns();//myColumn;
	prevV = presV-1;
	nextV = presV+1;
	topV = presV - mData->myPVColumns();
	lowerV = presV + mData->myPVColumns();
	
	if(mode == 0){
		JRow = presV;
		nextJRow = numPVvolumes;	
	}else{
		JRow = myVars*presV;
		nextJRow = 1;
	}
	
	for(int x = 0; x < 4; x++){
		// Cell Position Data Initialization
	myCV[x].myColumn = sData->getColumn();
	myCV[x].myRow = sData->getRow();
	myCV[x].myZrow = sData->getZrow();
	myCV[x].rowLength = mData->mysvLength();
	myCV[x].maxPvIndex = mData->myPVColumns()*mData->myPVRows()*mData->myPVZrows()-1;

	myCV[x].presV = /*eData->getPVstartColumn()+*/myColumn + myRow*mData->myPVColumns();//myColumn;
	myCV[x].prevV = presV-1;
	myCV[x].nextV = presV+1;
	myCV[x].topV = presV - mData->myPVColumns();
	myCV[x].lowerV = presV + mData->myPVColumns();
	
	if(mode == 0){
	myCV[x].JRow = presV;
	}else{
	myCV[x].JRow = myVars*presV;
	}
	}
	return;
}

void PV::initializeOtherPositions(void)
{	
	if(mode == 0){
		dn = 1;
		nv = numPVvolumes;
	}else{
		dn = myVars;
		nv = 1;
	}
	divc = presV;
	hpc = dn*presV;
	phpc = hpc-dn;
	nhpc = hpc+dn;
	thpc = dn*topV;
	bhpc = dn*lowerV;
	philc = dn*presV + nv;
	pphilc = philc - dn;
	nphilc = philc+dn;
	tphilc = dn*topV + nv;
	bphilc = dn*lowerV + nv;
	phisc = dn*presV + 2*nv;
	pphisc = phisc-dn;
	nphisc = phisc+dn;
	tphisc = dn*topV + 2*nv;
	bphisc = dn*lowerV + 2*nv;
	
	for(int x = 0; x < 4; x++){
	myCV[x].hpc = dn*presV;
	myCV[x].phpc = hpc-dn;
	myCV[x].nhpc = hpc+dn;
	myCV[x].thpc = dn*topV;
	myCV[x].bhpc = dn*lowerV;
	myCV[x].philc = dn*presV + nv;
	myCV[x].pphilc = philc - dn;
	myCV[x].nphilc = philc + dn;
	myCV[x].tphilc = dn*topV + nv;
	myCV[x].bphilc = dn*lowerV + nv;
	myCV[x].phisc = dn*presV + 2*nv;
	myCV[x].pphisc = phisc - dn;
	myCV[x].nphisc = phisc + dn;
	myCV[x].tphisc = dn*topV + 2*nv;
	myCV[x].bphisc = dn*lowerV + 2*nv;
	}
	return;
};
/*
void PV::initializeOtherPositions(void)
{
	divc = presV;
	hpc = presV;
	phpc = hpc-1;
	nhpc = hpc+1;
	thpc = topV;
	bhpc = lowerV;
	philc = presV + numPVvolumes;
	pphilc = philc - 1;
	nphilc = philc+1;
	tphilc = topV + numPVvolumes;
	bphilc = lowerV + numPVvolumes;
	phisc = presV + 2*numPVvolumes;
	pphisc = phisc-1;
	nphisc = phisc+1;
	tphisc = topV + 2*numPVvolumes;
	bphisc = lowerV + 2*numPVvolumes;
	
	for(int x = 0; x < 4; x++){
	myCV[x].hpc = presV;
	myCV[x].phpc = hpc-1;
	myCV[x].nhpc = hpc+1;
	myCV[x].thpc = topV;
	myCV[x].bhpc = lowerV;
	myCV[x].philc = presV + numPVvolumes;
	myCV[x].pphilc = philc - 1;
	myCV[x].nphilc = philc+1;
	myCV[x].tphilc = topV + numPVvolumes;
	myCV[x].bphilc = lowerV + numPVvolumes;
	myCV[x].phisc = presV + 2*numPVvolumes;
	myCV[x].pphisc = phisc-1;
	myCV[x].nphisc = phisc+1;
	myCV[x].tphisc = topV + 2*numPVvolumes;
	myCV[x].bphisc = lowerV + 2*numPVvolumes;
	}
	return;
};
*/

void PV::initializeVolumeChem(volumeChemData *vData)
{	
	phi_eq = vData->getphi_eq();
	hnot = vData->gethnot();
	mySign = vData->getmySign();
	return;
}

/*******************************************************/
/***************** Compute Volume Sizes ****************/
/*******************************************************/
void PV::initializeSize(void)
{
	initializeCVsizes();
	setMyVolume(&myVolume);
	tellCVPVvolume();
	return;	
}

void PV::initializeCVsizes(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      myCV[x].computedxSize();
      myCV[x].computedySize();
      myCV[x].computedzSize();
      myCV[x].computemyVolume();
    }
  }
  return;
};

void PV::setMyVolume(double* myV){
  myVolume = 0;
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      *myV += myCV[x].returnmyVolume();
    }
  }
  return;
}

void PV::tellCVPVvolume(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      myCV[x].setpvVolume(myVolume);
    }
  }
  return;
}
/*******************************************************/
/***************** Initialize mbScale ******************/
/*******************************************************/
void PV::initializembScale(void)
{
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      mbScale = myCV[x].returnmbScale();
    }
  }
  return;	
}

void PV::setInitialConditions(PVIConditions* iData)
{
  //	eps = iData->geteps();
  //	for(int x = 0; x < 4; x++){
  Vptr[hpc] = iData->gethp();
  //	cout << endl << "Assigned hpc = " << Vptr[hpc];
  //	}
  Vptr[philc] = iData->getphi_l();
  Vptr[phisc] = iData->getphi_s();
  //	Vptr[jlxc] = iData->getjlx();
  return;
}

void PV::computeDerivatives(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      myCV[x].setdhdt();
    }
  }
  return;
}
/****************************************************/
/*************** Setup FV Pointers ******************/
/****************************************************/
void PV::setupFvPointers(FV** myFv){
  if(myCV[UL].exist)
    setULFV(myFv,&myCV[UL]);
  if(myCV[UR].exist)
    setURFV(myFv,&myCV[UR]);
  if(myCV[LL].exist)
    setLLFV(myFv,&myCV[LL]);
  if(myCV[LR].exist)
    setLRFV(myFv,&myCV[LR]);
  return;
};

void PV::setULFV(FV** FVp, CV* myCV){
  myCV->myFV = FVp[(myRow-1)*numFVColumns+(myColumn-1)];
  return;
};

void PV::setURFV(FV** FVp, CV* myCV){
  myCV->myFV = FVp[(myRow-1)*numFVColumns+(myColumn)];
  return;
};

void PV::setLLFV(FV** FVp, CV* myCV){
  myCV->myFV = FVp[(myRow)*numFVColumns+(myColumn-1)];
  return;
};

void PV::setLRFV(FV** FVp, CV* myCV){
  myCV->myFV = FVp[(myRow)*numFVColumns+(myColumn)];
  return;
};


void PV::setupJacobian()
{
  JRowBuffer = JRow; 
  //	JRow += myVols*rowLength;
  //	hpJ();
  //	JRow += myVols*rowLength;
  //	phisJ();
  //	JRow += myVols*rowLength;
  //	jlxJ();
  //	JRow += myVols*rowLength;
  //	philJ();
  //	JRow += myVols*rowLength;
  //	jlyJ();
  //	JRow += myVols*rowLength;
  ///	 epsJ();
  ///	 JRow += myVols*rowLength;
  mbJ();
  JRow += nextJRow;
  docJ();
  JRow += nextJRow;
  ekJ();
  JRow = JRowBuffer;
  return;
};


/************************************************************/
/*********** Contribution from Each of the 4 X volumes ******/
/*********** Material Balance Equation **********************/
/************************************************************/

/*******************************************************/
/********** Compute Equations **************************/
/*******************************************************/

void PV::computeIValues(void){
  computeIVmb();
  computeIVdoc();
  computeIVek();
  return;
};

void PV::computeIVmb(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      iVptr[hpc] += myCV[x].setdhdtCV() + mbScale*myCV[x].setdocCVjl();
    }
  }
  return;
};

void PV::computeIVdoc(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      iVptr[philc] += myCV[x].setdocCV();
    }
  }
  return;
};

void PV::computeIVek(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      iVptr[phisc] += myCV[x].setdocCVjl() + myCV[x].setekCV();
    }
  }
  return;
};

void PV::computeDiv(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      divVptr[divc] += myCV[x].setdocCVjl();
    }
  }
  return;
};
/******************************************************/
/************* EK Jacobian Equations *****************/
/******************************************************/
void PV::ekJ(void){
  docJtphil(1);
  docJpphil(1);
  docJphil(1);
  docJnphil(1);
  docJbphil(1);
  
  docJthp(1);
  docJphp(1);
  docJhp(1);
  docJnhp(1);
  docJbhp(1);
  
  //ekJeps();
  ekJhp();
  ekJphis();
  ekJphil();
  return;
};

void PV::ekJeps(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      //	Jacobptr[JRow][epsc] = Jacobptr[JRow][epsc] + myCV[x].ekJeps();
    }
  }
  return;
};

void PV::ekJhp(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      Jacobptr[JRow][hpc] += myCV[x].ekJhp();
    }
  }
  return;
};

void PV::ekJphis(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      Jacobptr[JRow][phisc] += myCV[x].ekJphis();
    }
  }
  return;
};

void PV::ekJphil(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      Jacobptr[JRow][philc] += myCV[x].ekJphil();
    }
  }
  return;
};

/******************************************************/
/************* DOC Jacobian Equations *****************/
/******************************************************/
void PV::docJ(void){
  docJtphil(1);
  docJpphil(1);
  docJphil(1);
  docJnphil(1);
  docJbphil(1);
  
  docJthp(1);
  docJphp(1);
  docJhp(1);
  docJnhp(1);
  docJbhp(1);
  
  docJtphis();
  docJpphis();
  docJphis();
  docJnphis();
  docJbphis();
  return;
};
/*****************************************************************/
/********* DOC Jacobian Phil *************************************/
/*****************************************************************/
void PV::docJtphil(double scale){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][tphilc] += scale*myCV[UL].myJyphil(); 
  }
  if(myCV[UR].exist){
    Jacobptr[JRow][tphilc] += scale*myCV[UR].myJyphil(); 
  }
  return;
};

void PV::docJpphil(double scale){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][pphilc] += scale*myCV[UL].myJxphil(); 
  }
  if(myCV[LL].exist){
    Jacobptr[JRow][pphilc] += scale*myCV[LL].myJxphil(); 
  }
  return;
};

void PV::docJphil(double scale){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      Jacobptr[JRow][philc] += scale*myCV[x].myJphil();
    }
  }
  return;
};

void PV::docJnphil(double scale){
  if(myCV[UR].exist){
    Jacobptr[JRow][nphilc] += scale*myCV[UR].myJxphil(); 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][nphilc] += scale*myCV[LR].myJxphil(); 
  }
  return;
};

void PV::docJbphil(double scale){ 
  if(myCV[LL].exist){
    Jacobptr[JRow][bphilc] += scale*myCV[LL].myJyphil(); 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][bphilc] += scale*myCV[LR].myJyphil(); 
  }
  return;
};

/***********************************************************/
/************** doc Jacobian hp terms **********************/
/***********************************************************/
void PV::docJthp(double scale){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][thpc] += (scale*myCV[UL].myDOCJyhp())/Vptr[thpc]; 
  }
  if(myCV[UR].exist){
    Jacobptr[JRow][thpc] += (scale*myCV[UR].myDOCJyhp())/Vptr[thpc]; 
  }
  return;
};

void PV::docJphp(double scale){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][phpc] += (scale*myCV[UL].myDOCJxhp())/Vptr[phpc]; 
  }
  if(myCV[LL].exist){
    Jacobptr[JRow][phpc] += (scale*myCV[LL].myDOCJxhp())/Vptr[phpc]; 
  }
  return;
};

void PV::docJhp(double scale){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      Jacobptr[JRow][hpc] += (scale*myCV[x].myDOCJhp())/Vptr[hpc];
    }
  }
  return;
};

void PV::docJnhp(double scale){
  if(myCV[UR].exist){
    Jacobptr[JRow][nhpc] += (scale*myCV[UR].myDOCJxhp())/Vptr[nhpc]; 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][nhpc] += (scale*myCV[LR].myDOCJxhp())/Vptr[nhpc]; 
  }
  return;
};

void PV::docJbhp(double scale){ 
  if(myCV[LL].exist){
    Jacobptr[JRow][bhpc] += (scale*myCV[LL].myDOCJyhp())/Vptr[bhpc]; 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][bhpc] += (scale*myCV[LR].myDOCJyhp())/Vptr[bhpc]; 
  }
  return;
};

/***********************************************************/
/************** doc Jacobian phis terms ********************/
/***********************************************************/
void PV::docJtphis(void){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][tphisc] += myCV[UL].myJyphis(); 
  }
  if(myCV[UR].exist){
    Jacobptr[JRow][tphisc] += myCV[UR].myJyphis(); 
  }
  return;
};

void PV::docJpphis(void){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][pphisc] += myCV[UL].myJxphis(); 
  }
  if(myCV[LL].exist){
    Jacobptr[JRow][pphisc] += myCV[LL].myJxphis(); 
  }
  return;
};

void PV::docJphis(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      Jacobptr[JRow][phisc] += myCV[x].myJphis();
    }
  }
  return;
};

void PV::docJnphis(void){
  if(myCV[UR].exist){
    Jacobptr[JRow][nphisc] += myCV[UR].myJxphis(); 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][nphisc] += myCV[LR].myJxphis(); 
  }
  return;
};

void PV::docJbphis(void){ 
  if(myCV[LL].exist){
    Jacobptr[JRow][bphisc] += myCV[LL].myJyphis(); 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][bphisc] += myCV[LR].myJyphis(); 
  }
  return;
};
/******************************************************/
/**********Material Balance Jacobian Equations ********/
/******************************************************/
void PV::mbJ(void){
  mbJthp();
  mbJphp();
  mbJhp();
  mbJnhp();
  mbJbhp();
  
  docJtphil(mbScale);
  docJpphil(mbScale);
  docJphil(mbScale);
  docJnphil(mbScale);
  docJbphil(mbScale);
  
  docJthp(mbScale);
  docJphp(mbScale);
  docJhp(mbScale);
  docJnhp(mbScale);
  docJbhp(mbScale);
  return;
};

void PV::mbJthp(void){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][thpc] += myCV[UL].myJyhp(); 
  }
  if(myCV[UR].exist){
    Jacobptr[JRow][thpc] += myCV[UR].myJyhp(); 
  }
  return;
};

void PV::mbJphp(void){ 
  if(myCV[UL].exist){
    Jacobptr[JRow][phpc] += myCV[UL].myJxhp(); 
  }
  if(myCV[LL].exist){
    Jacobptr[JRow][phpc] += myCV[LL].myJxhp(); 
  }
  return;
};

void PV::mbJhp(void){
  for(int x = 0; x < 4; x++){
    if(myCV[x].exist){
      Jacobptr[JRow][hpc] += myCV[x].myJhp();
    }
  }
  return;
};

void PV::mbJnhp(void){
  if(myCV[UR].exist){
    Jacobptr[JRow][nhpc] += myCV[UR].myJxhp(); 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][nhpc] += myCV[LR].myJxhp(); 
  }
  return;
};

void PV::mbJbhp(void){ 
  if(myCV[LL].exist){
    Jacobptr[JRow][bhpc] += myCV[LL].myJyhp(); 
  }
  if(myCV[LR].exist){
    Jacobptr[JRow][bhpc] += myCV[LR].myJyhp(); 
  }
  return;
};
