/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* @author James Geraci
 */
#pragma once
#include "FV.h"
#include "CV.h"

class PV
{
public:
	PV(void);
	PV(volumeSpatialData*,matrixData*,volumeChemData*,electrodeSizeData*,PVIConditions*);
public:
	virtual ~PV(void);

	void computeDerivatives(void);
	void computeIValues(void);
	void setupJacobian(void);
	void setupFvPointers(FV** myFv);
	void initializeSize(void);
	void initializembScale(void);
	void computeDiv(void);
	
protected:

	CV myCV[4];

	// Data Matrix Pointers
	double* fluxVptr;
	double* divVptr;
	double* pVptr;
	double* Vptr;
	double* iVptr;
	double** Jacobptr;

	// Functions
	void initializeMatrix(matrixData*);
	void initializePosition(volumeSpatialData*, matrixData*, electrodeSizeData*);
	void initializeOtherPositions(void);
	// void initializeSize(volumeSpatialData*);
	void setInitialConditions(PVIConditions*);
	void initializeVolumeChem(volumeChemData*);
	void initializeCVsizes(void);
	void setMyVolume(double*);
	void tellCVPVvolume(void);
	// 2d model
	void setURFV(FV**,CV*);
	void setULFV(FV**,CV*);
	void setLLFV(FV**,CV*);
	void setLRFV(FV**,CV*);

	void computeIVmb(void);
	virtual void computeIVdoc(void);
	void computeIVek(void);

	// Electrode Kinetics Jacobian Equations
	void ekJ(void);
	void ekJeps(void);
	void ekJhp(void);
	void ekJphis(void);
	void ekJphil(void);

	// Divergence of Current Jacobian Equations
	void docJ(void);	
	void docJtphil(double);
	void docJpphil(double);
	void docJphil(double);
	void docJnphil(double);
	void docJbphil(double);

	void docJthp(double);
	void docJphp(double);
	void docJhp(double);
	void docJnhp(double);
	void docJbhp(double);

	void docJtphis(void);
	void docJpphis(void);
	void docJphis(void);
	void docJnphis(void);
	void docJbphis(void);
	// Material Balance Jacobian Equations
	void mbJ(void);
	void mbJthp(void);
	void mbJphp(void);
	void mbJhp(void);
	void mbJnhp(void);
	void mbJbhp(void);

	// Time Derivatives
	double dhdt;
	double setdhdt(void);

	// Position information for Cell
	int myColumn;
	int myRow;
	int myZrow;
	int JRow;
	int JRowBuffer;

    // Positioning Data
	int rowLength;
	int maxPvIndex;
	int nextJRow;

	// FV grid Information
	int numFVColumns;
	int numFVRows;

	//int maxPvIndex;
	int numPVvolumes;
	int dn;
	int nv;

	//  Used in Calculating Addresses
	int prevV;
	int presV;
	int nextV;

	int topV;
	int lowerV;
	// present volume's properties
	int hpc;
	int phisc;
	int philc;
// previous volume's properties
	int phpc;
	int pphisc;
	int pphilc;
// next volume's properties
	int nhpc;
	int nphisc;
	int nphilc;
// top volume's properties
	int thpc;
	int tphisc;
	int tphilc;
// lower volume's properties
	int bhpc;
	int bphisc;
	int bphilc;

	int divc;
	double phi_eq;
	double mySign;
	double hnot;
	double mbScale;
	double myVolume;
};

