/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/** 
 * cellSizeData.cpp 
 * @author James Geraci
 * provides the interface to set/fetch the number of columns and rows  
 * in each region of the battery cell
 *
 */
#include "StdAfx.h"
#include "cellSizeData.h"

cellSizeData::cellSizeData(void)
{
}

cellSizeData::~cellSizeData(void)
{
}


void cellSizeData::setnumofElectrodes(int nE){
	numofElectrodes = nE;
	return;
};

void cellSizeData::setldxFVColumns(int ldxFVc){
	ldxFVColumns = ldxFVc;
	return;
};

void cellSizeData::setElectrolyteFVColumns(int eFVc){
	ElectrolyteFVColumns = eFVc;
	return;
};

void cellSizeData::setldFVColumns(int ldFVc){
	ldFVColumns = ldFVc;
	return;
};

void cellSizeData::setldxPVColumns(int ldxPVc){
	ldxPVColumns = ldxPVc;
	return;
};

void cellSizeData::setElectrolytePVColumns(int ePVc){
	ElectrolytePVColumns = ePVc;
	return;
};

void cellSizeData::setldPVColumns(int ldPVc){
	ldPVColumns = ldPVc;
	return;
};


void cellSizeData::setnumFVRows(int FVrows){
	numFVRows = FVrows;
	return;
};

void cellSizeData::setnumPVRows(int PVrows){
	numPVRows = PVrows;
	return;
};


void cellSizeData::setnumFVZrows(int FVZrows){
	numFVZrows = FVZrows;
	return;
};

void cellSizeData::setnumPVZrows(int PVZrows){
	numPVZrows = PVZrows;
	return;
};

void cellSizeData::computeTotalFVColumns(void){
	totalFVColumns = ldxFVColumns + ElectrolyteFVColumns + ldFVColumns;
	return;
};

void cellSizeData::computeTotalPVColumns(void){
	totalPVColumns = ldxPVColumns + ElectrolytePVColumns + ldPVColumns;
	return;
};

void cellSizeData::computeTotalFVVolumes(void){
	totalFVVolumes = totalFVColumns*numFVRows*numFVZrows;
	return;
};

void cellSizeData::computeTotalPVVolumes(void){
	totalPVVolumes = totalPVColumns*numPVRows*numPVZrows;
	return;
};


int cellSizeData::getnumPVZrows(void){
	return numPVZrows;	
};

int cellSizeData::getnumPVRows(void){
	return numPVRows;	
};

int cellSizeData::getnumPVColumns(void){
	return totalPVColumns;	
};


int cellSizeData::getnumFVZrows(void){
	return numFVZrows;	
};

int cellSizeData::getnumFVRows(void){
	return numFVRows;	
};

int cellSizeData::getnumFVColumns(void){
	return totalFVColumns;	
};


int cellSizeData::gettotalFVVolumes(void){
	return totalFVVolumes;	
};


int cellSizeData::gettotalPVVolumes(void){
	return totalPVVolumes;	
};

int cellSizeData::getnumofElectrodes(void){
	return numofElectrodes;
};

int cellSizeData::getldxFVColumns(void){
	return ldxFVColumns;
};

int cellSizeData::getElectrolyteFVColumns(void){
	return  ElectrolyteFVColumns;
};

int cellSizeData::getldFVColumns(void){
	return ldFVColumns;
};

int cellSizeData::getldxPVColumns(void){
	return ldxPVColumns;
};

int cellSizeData::getElectrolytePVColumns(void){
	return ElectrolytePVColumns;
};

int cellSizeData::getldPVColumns(void){
	return ldPVColumns;
};

