/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* @author James Geraci
 */
/** 
 * electrodeChemData.cpp
 * Generic to any electrode
 * @author James Geraci
 * Set/Fectch some physical properties for each electrode
 */
#include "StdAfx.h"
#include "electrodeChemData.h"

const double electrodeChemData::PbSO4_MW = 303.2626;  // g
const double electrodeChemData::PbSO4_rho = 6.39; // g/cm^3


electrodeChemData::electrodeChemData(void)
{
}

electrodeChemData::electrodeChemData(double Tnp, double io, double mbscale, double vf)
{
	ecDataInit(Tnp,io,mbscale,vf);
}


electrodeChemData::~electrodeChemData(void)
{
}

void electrodeChemData::ecDataInit(double Tnp, double io, double mbS, double vf)
{
	settnp(Tnp);
	setmbScale(mbS);
	setolsScale();
	setmyio(io);
	setmyVf(vf);
	return;
}

void electrodeChemData::settnp(double Tnp)
{
	myTnp = Tnp;
	return;
};

void electrodeChemData::setmbScale(double mb)
{
	mbScale = mb;
	return;
};

void electrodeChemData::setolsScale()
{
	olsScale = -(1-2*myTnp)*(R*298)/F;
	return;
};

void electrodeChemData::setmyio(double io)
{
	myio = io;
	return;
};

void electrodeChemData::setmyVf(double vf)
{
	myVf = vf;
	return;
};

double electrodeChemData::getmbScale(void)
{
	return mbScale;
}

double electrodeChemData::getolsScale(void)
{
	return olsScale;
}

double electrodeChemData::gettnp(void)
{
	return myTnp;
}

double electrodeChemData::getmyVf(void)
{
	return myVf;
}

double electrodeChemData::getio(void)
{
	return myio;
}
