/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* 
 * @author James Geraci
 * Used in the creating of electrodes to set the physical sizes and
 * grid sizes and how they relate to each other
 */
#include "StdAfx.h"
#include "electrodeSizeData.h"
#include "cellSizeData.h"

electrodeSizeData::electrodeSizeData(void)
{
	numFVZrows = 1;
}

electrodeSizeData::~electrodeSizeData(void)
{
}

electrodeSizeData::electrodeSizeData(double w, double h, double d, int FVc, int FVr, int FVz,
									 int PVc, int PVr, int PVz, int FVsr, int FVsc, int PVsr, int PVsc, cellSizeData* csData){
	xWidth = w;
	yHeight = h;
	zDepth = d;
	numFVColumns = FVc;
	numFVRows = FVr;
	numFVZrows = FVz;
	numPVColumns = PVc;
	numPVRows = PVr;
	numPVZrows = PVz;
	FVstartRow = FVsr;
	FVstartColumn = FVsc;
	
	PVstartRow = PVsr;
	PVstartColumn = PVsc;
	mycsData = csData;
}
void electrodeSizeData::setFVColumns(int nColumns)
{
	numFVColumns = nColumns;
	return;
};

void electrodeSizeData::setFVRows(int nRows)
{
	numFVRows = nRows;
	return;
};

void electrodeSizeData::setFVZrows(int nZrows)
{
	numFVZrows = nZrows;
	return;
};

void electrodeSizeData::setFVStartColumn(int sColumn)
{
	FVstartColumn = sColumn;
	return;
};

void electrodeSizeData::setyHeight(double H)
{
	yHeight = H;
	return;
};

void electrodeSizeData::setxWidth(double W)
{
	xWidth = W;
	return;
};

void electrodeSizeData::setZdepth(double D)
{
	zDepth = D;
	return;
};


int electrodeSizeData::getnumFVColumns(void)
{
	return numFVColumns;
};

int electrodeSizeData::getnumFVRows(void)
{
	return numFVRows;
};

int electrodeSizeData::getnumFVZrows(void)
{
	return numFVZrows;
};


int electrodeSizeData::getnumPVColumns(void)
{
	return numPVColumns;
};

int electrodeSizeData::getnumPVRows(void)
{
	return numPVRows;
};

int electrodeSizeData::getnumPVZrows(void)
{
	return numPVZrows;
};

double electrodeSizeData::getyHeight(void)
{
	return yHeight;
};

double electrodeSizeData::getxWidth(void)
{
	return xWidth;
};

double electrodeSizeData::getzDepth(void)
{
	return zDepth;
};

int electrodeSizeData::getFVstartColumn(void)
{
	return FVstartColumn;
};

int electrodeSizeData::getFVstartRow(void)
{
	return FVstartRow;
};


int electrodeSizeData::getPVstartColumn(void)
{
	return PVstartColumn;
};

int electrodeSizeData::getPVstartRow(void)
{
	return PVstartRow;
};
