/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* @author James Geraci
 */
#ifndef _MATRIXDATA_H
#define _MATRIXDATA_H
#include "cellSizeData.h"
#include <stdio.h>

#include <libspe.h>
#include "common.h"
class matrixData
{
public:
	matrixData(void);
	matrixData(cellSizeData*);//int,int,int);
public:
	~matrixData(void);

public:
	FILE* Vfile;
	void writeVtoDisk(double*,int);

	// Sparse Matrix Represenation Structures
	//	double *val_CCS, *row_ind, *col_ptr;
	double *val_CRS, *col_ind, *row_ptr;
	int sparseMatrixSize;
	int ActualsparseMatrixSize;
	unsigned int *nzInRow;
	double** myJacobian(void);
	double* myfluxV(void);
	double* mysocV(void);
	double* mydivV(void);
	double* myepsV(void);
	double* myiV(void);
	double* mysV(void);
	double* mypiV(void);
	double* mypsV(void);
	double* myRowBuffer(void);
	double* myExtraBuffer1(void);
	int* myintBuffer(void);
	
	int myJacobByteSize(void);
	int myJacobSize(void);
	int mysvLength(void);
	int mySvByteSize(void);

	int myPVColumns(void);
	int myPVRows(void);
	int myPVZrows(void);

	int myFVColumns(void);
	int myFVRows(void);
	int myFVZrows(void);
	double* myJacobianStorage(void);
	void setpiV(void);
	void resetiV(void);
	void resetdivV(void);
	void resetJacobian(void);
	void resetRowBuffer(void);
	void resetFluxes(void);
	void restoresV(void);
	void storesV(void);
	
	/* void cpJacobiantoMatlab(void); */
/* 	void cpurJacobiantoMatlab(void); */
/* 	void cpIVtoMatlab(void); */
/* 	void cpSVtoMatlab(void); */
/* 	void cppSVtoMatlab(void); */
	void setInitialSV(void);
/* 	void cpfluxVtoMatlab(void); */
/* 	void cpdivVtoMatlab(void); */
/* 	void cpepsVtoMatlab(void); */
/* 	void cpsocVtoMatlab(void);*\/ */
	void applyBoundaryConditions(void);
	void applyPreConditioner(void);
/* 	void getSVfromMatlab(void); */
/* 	void getIVfromMatlab(void); */
/* 	void computnIVinMatlab(void); */
/* 	void makeGraphs(void); */
	void computeIV(void);
	//void computeIV(speid_t*, double*,double*);

	speid_t* ids;
	double* rhs;
	int sum;
	int present_base_row;
	double rhsBuffer;
	double* bufaddrf;
	int svLength; // = numColumns*numRows*6;
	int hbWidth;
	int hbWbytes;
	double* JacobianStorage;
	double** Jacobian;
	double* sV;
	double* iV;  // intermediate values vector
private:
	double tic;
	double toc;
	double elapsed;
	bool first_call;
	//	Engine *ep;
	//MATFile *pmat;
	//	mxArray *fluxVmat, *divVmat, *epsVmat, *socVmat, *psVmat, *sVmat, *iVmat, *savesVmat, *piVmat, *rowBuffermat, *JacobianMat;
	double* fluxV;
	double* divV;
	double* epsV;
	double* socV;
	double* psV;
	
	double* savesV;

	double* piV; // the Past difference
	double* rowBuffer;
	double* extraBuffer1;
	
	int* intBuffer;
	
	double max_pivot_value;
	int max_pivot;
	
	int svByteSize;
	int FVsvLength;
	int FVsvByteSize;
	int JacobSize;
	int JacobByteSize;
	int eyeMaker;
	
	int ldxPVColumns;
	int ElectrolytePVColumns;
	int ldPVColumns;

	int f;

	int numPVColumns;
	int numPVRows;
	int numPVZrows;
	int totalPVvolumes;

	int numFVColumns;
	int numFVRows;
	int numFVZrows;
	int totalFVvolumes;
};


#endif
