/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* @author James Geraci
 */

//  James Geraci
// 6.189 MultiCoreProgramming
// January 23, 2007

#include <spu_mfcio.h>
#include <libmisc.h>
#include <stdio.h>
#include <spu_intrinsics.h>
#include <vec_literal.h>
#include <string.h>
#include <simdmath/divd2.h>
#include <sys/time.h>
#include "common.h"

static  CONTROL_BLOCK_T cb[NUM_SPU] QWORD_ALIGNED;  

int my_rank;
int number_of_full_dmas_needed;
int last_dma_size;

uint32_t* row_pointers;
uint32_t* rhs_row_pointers;
int two_byteSize;
double base_row_factor;
double elim_row_factor;

vector double vFactor;
int present_base_row;
int present_elim_row;
int future_elim_row;
int present_elim_column;
int nbr;
double adjustFactor;
int n;
int vector_n;
int n_byteSize;
int hbW;
int hbWfull;
int hbWbyteSize;
int hbWbufferByteSize;
int last_base_row;

static inline void
get_data(void *row_buf, void *rhs_buf, uint32_t row, uint32_t tag, uint32_t pbr){
  //  memset(row_buf,0,hbWbufferByteSize);
  if(pbr < n - hbWfull){
    if(pbr%2){
      mfc_get((char*) row_buf, row_pointers[row]+(pbr-1)*sizeof(double), hbWfull*sizeof(double),tag,0,0);
    }else{
      mfc_get((char*) row_buf, row_pointers[row] + pbr*sizeof(double), (hbWfull)*sizeof(double),tag,0,0);
    }
  }else{
    if(pbr%2){
      mfc_get((char*) row_buf, row_pointers[row]+(pbr-1)*sizeof(double), (n-(pbr-1))*sizeof(double),tag,0,0);
    }else{
      mfc_get((char*) row_buf, row_pointers[row] + pbr*sizeof(double), (n-pbr)*sizeof(double),tag,0,0);
    }
  }
  mfc_get(rhs_buf, rhs_row_pointers[row], two_byteSize, tag, 0, 0);
}

static inline void
put_data(void *row_buf, void *rhs_buf, uint32_t row, uint32_t tag, uint32_t pbr){
  if(pbr < n-hbWfull){
    if(pbr%2){
      mfc_put((char*) row_buf, row_pointers[row]+(pbr-1)*sizeof(double), hbWfull*sizeof(double),tag,0,0);
    }else{
      mfc_put((char*) row_buf, row_pointers[row] + pbr*sizeof(double), (hbWfull)*sizeof(double),tag,0,0);
    }
  }else{
    if(pbr%2){
      mfc_put((char*) row_buf, row_pointers[row]+(pbr-1)*sizeof(double), (n-(pbr-1))*sizeof(double),tag,0,0);
    }else{
      mfc_put((char*) row_buf, row_pointers[row] + pbr*sizeof(double), (n-pbr)*sizeof(double),tag,0,0);
    }
  }
  mfc_put(rhs_buf, rhs_row_pointers[row], two_byteSize, tag, 0, 0);
}

static inline void
compute_data(vector double* b_row_buf, vector double* e_row_buf, double b_rhs, double* e_rhs, int pbr){
  //COMPUTE
  present_elim_column = 0;
  if(!(pbr%2)){
    elim_row_factor = spu_extract(e_row_buf[present_elim_column],0);
    
  }else{
    elim_row_factor = spu_extract(e_row_buf[present_elim_column],1);
  }
  adjustFactor = elim_row_factor/base_row_factor;
  // set up for SIMD multiplying
  vFactor = spu_insert(adjustFactor,vFactor,0);
  vFactor = spu_insert(adjustFactor,vFactor,1);
  for(; present_elim_column <  hbWfull/2; present_elim_column++){
      e_row_buf[present_elim_column] = spu_nmsub(vFactor,b_row_buf[present_elim_column],e_row_buf[present_elim_column]);
    }
  *e_rhs -= adjustFactor * b_rhs;
  //printf("Computed Row %d on SPU %d \n",present_elim_row, my_rank);
  //COMPUTE_END
}
   

int main(uint64_t speid, uint64_t argp, uint64_t envp)
{
  int ppu_sent_next_base_row = 0;
  nbr = 0;
  adjustFactor = 0;
  
  spu_write_out_mbox(1);  //  Tell the PPU that we are here

  my_rank = spu_read_in_mbox();  // Wait for all the SPU to catch up
  
  mfc_get(cb, argp, sizeof(cb),0,0,0);
  mfc_write_tag_mask(1<<0);
  mfc_read_tag_status_all();
  
  n = cb[my_rank].svLength_size;
  vector_n = n/2;
  n_byteSize = n*sizeof(double);
  two_byteSize = 2*sizeof(double);
  hbW = cb[my_rank].hbWidth;
  last_base_row = n-1;
  if(hbW%2){
    hbWfull = hbW+1;
  }else{
    hbWfull = hbW+2;
  }
  hbWbufferByteSize = (hbWfull + 0)*sizeof(double);
  row_pointers = reinterpret_cast<uint32_t*>(malloc_align(sizeof(uint32_t)*n,7));
  rhs_row_pointers = reinterpret_cast<uint32_t*>(malloc_align(sizeof(uint32_t)*n,7));
  uint32_t rowSwapBuffer;

  vector double* base_row = reinterpret_cast<vector double*>(malloc_align(hbWbufferByteSize,7));
  vector double* elim_row = reinterpret_cast<vector double*>(malloc_align(hbWbufferByteSize,7));
  vector double* next_elim_row = reinterpret_cast<vector double*>(malloc_align(hbWbufferByteSize,7));
 

  double* base_rhs = reinterpret_cast<double*>(malloc_align(two_byteSize,7));
  double* elim_rhs = reinterpret_cast<double*>(malloc_align(two_byteSize,7));
  double* next_elim_rhs = reinterpret_cast<double*>(malloc_align(two_byteSize,7));
 
  vector double* temp_elim_row;
  double* temp_elim_rhs;
  int hbPbr;
  number_of_full_dmas_needed = (int) n_byteSize/MAX_BYTES_IN_ONE_DMA;
  last_dma_size = n_byteSize - MAX_BYTES_IN_ONE_DMA*(number_of_full_dmas_needed);    

  for(int row = 0; row < n; row++){
    row_pointers[row] = (uint32_t) cb[my_rank].matrixData_addr+row*n_byteSize;
    rhs_row_pointers[row] = (uint32_t) cb[my_rank].rhs_addr+row*two_byteSize;
  }

  while(1){
    for(present_base_row = 0; present_base_row < last_base_row; present_base_row++){
      nbr = present_base_row + 1;
      hbPbr = present_base_row + hbWfull;
      get_data(&base_row[0],base_rhs,present_base_row,0,present_base_row);
      present_elim_row = present_base_row + 1 + my_rank;
      
      if (present_elim_row <= last_base_row){
     	//FETCH FIRST
	get_data(elim_row, elim_rhs, present_elim_row,0,present_base_row);
	//FETCH_END
	mfc_read_tag_status_all(); //BARRIER
	
	// Unpack the base row scale factor
	if(!(present_base_row%2)){
	  base_row_factor = spu_extract(base_row[0],0);
	}else{base_row_factor = spu_extract(base_row[0],1);
	}
	// Unpack the base row scale factor
	//if(!(present_base_row%2)){
	//  base_row_factor = spu_extract(base_row[present_base_row/2],0);
	//	}else{base_row_factor = spu_extract(base_row[(present_base_row-1)/2],1);
	//	}
	
	for(; present_elim_row+NUM_SPU <=last_base_row && present_elim_row+NUM_SPU <= hbPbr; present_elim_row += NUM_SPU){
	  
	  future_elim_row = present_elim_row + NUM_SPU;
	  //FETCH
	  get_data(next_elim_row,next_elim_rhs,future_elim_row,0,present_base_row);
	  //printf("Fetching Row %d on SPU %d \n",future_elim_row, my_rank);
	  //FETCH_END
	  
	  compute_data(base_row,elim_row,base_rhs[0],elim_rhs,present_base_row);
	  
	  //SEND
	  put_data(elim_row,elim_rhs,present_elim_row,0,present_base_row);	   
	  //printf("Sending Row %d on SPU %d \n",present_elim_row, my_rank);
	  //SEND_END
	  //	  if(my_rank == 0 && mfc_stat_cmd_queue() == 0)
	  //  printf("My queue length = %d\n", mfc_stat_cmd_queue());
	  
	  mfc_read_tag_status_all(); //BARRIER
	  temp_elim_row = next_elim_row;
	  temp_elim_rhs = next_elim_rhs;
	  next_elim_row = elim_row;
	  next_elim_rhs = elim_rhs;
	  elim_row = temp_elim_row;
	  elim_rhs = temp_elim_rhs;  
	} // END of for-loop elimination
	
	compute_data(base_row,elim_row,base_rhs[0],elim_rhs,present_base_row);
	
	//SEND FINAL
	put_data(elim_row,elim_rhs,present_elim_row,0,present_base_row);
	//printf("Sending Last Row %d on SPU %d \n",present_elim_row, my_rank);
	//SEND_END
	
	mfc_read_tag_status_all(); //BARRIER
	
	
      } //END of if (present_elim_row <= last_base_row)
      //Thus, Done Eliminating all the Rows Allocated to this SPU
      //printf("SPU %d got through on iteration of base_row %d\n",my_rank, present_base_row); 
      spu_write_out_mbox(1);
      
         ppu_sent_next_base_row = spu_read_in_mbox();

      
	 if(ppu_sent_next_base_row != present_base_row + 1)
      	{
      	  rowSwapBuffer = row_pointers[ppu_sent_next_base_row];
      	  row_pointers[ppu_sent_next_base_row] = row_pointers[present_base_row + 1];
      	  row_pointers[present_base_row + 1] = rowSwapBuffer;
      	}
	 
    } // ENDS base row loop
    
    spu_read_in_mbox();
    
  } // Ends Major While loop
  return 0;
}
 
