% Copyright (c) 2007 Massachusetts Institute of Technology
%
% Permission is hereby granted, free of charge, to any person obtaining a copy of
% this software and associated documentation files (the "Software"), to deal in
% the Software without restriction, including without limitation the rights to
% use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
% the Software, and to permit persons to whom the Software is furnished to do so,
% subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
% FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
% COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
% IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
% CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.
%
% Author: James Geraci
% April 26, 2007
% Tool for extracting Parameters from PS3 .dat file
%
% svDataTool takes three arguments, the number of PV columns
% the number of PV Row, and the timeStep that you
% wish to plot
% For example, if you created your model with
%   csData->setldxFVColumns(11);
%   csData->setElectrolyteFVColumns(11);
%   csData->setldFVColumns(11);
%   csData->setnumFVRows(31);
% and you wanted to plot the data from time step 0
% you would need the file mySV_0.dat in the same directory as
% svDAtaTool.m
% then typing:
% > svDataTool(34,32,0)
% at the command prompt will extract and plot the data
% from the first time step
% while typing
% > svDataTool(34,32,10)
% at the command prompt will extract and plot the data from
% the 11th time step if mySV_10.dat is also in the directory.

function svDataTool(numColumns, numRows, TimeStep)
%numColumns = 34;  % Total number of PV Columns
%numRows = 32;  % Total number of PV Rows
numValues = 3;  % This never Changes
SVsize = numColumns*numRows*numValues; % Size of SV vector

c = SVsize*8;
%Data script
datFileName = strcat(strcat('mySV_',num2str(TimeStep)),'.dat');  % File you wish to extract Data from
fid = fopen(datFileName,'r','s');
A = fread(fid,c,'double',0,'s');
fclose all

% Reshape data to a matrix of shape
An0 = reshape(A,numValues,numRows*numColumns);

% extract concentration data
Conc = An0(1,:)
x = 1:numColumns;
y = 1:numRows;
Conc = reshape(Conc,numColumns,numRows);
figure
mesh(y,x,Conc)
xlabel('Height in Volumes')
ylabel('Width in Volumes')
title('Concentration Profile')
zlabel('Concentration')

% extract the phil data
phil = An0(2,:)
phil = reshape(phil,numColumns,numRows);
figure
mesh(y,x,phil)
xlabel('Height in Volumes')
ylabel('Width in Volumes')
title('Liquid Phase Potential Profile')
zlabel('Potential')

% extract the phil data
phis = An0(3,:)
phis = reshape(phis,numColumns,numRows);
figure
mesh(y,x,phis)
xlabel('Height in Volumes')
ylabel('Width in Volumes')
title('Solid Phase Potential Profile')
zlabel('Potential')

% plot the overvoltage profile
figure
mesh(y,x,phis-phil)
xlabel('Height in Volumes')
ylabel('Width in Volumes')
title('Overvoltage Profile')
zlabel('Potential')