/* Copyright (c) 2007 Massachusetts Institute of Technology
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* @author James Geraci
 */
#ifndef _VOLUMECHEMDATA_H
#define _VOLUMECHEMDATA_H

class volumeChemData
{
public:
	volumeChemData(void);
	volumeChemData(double,double,double,double,double,double,double,double,double,double,
		double,double,double,double,double,double,double,double,double,double,double);
public:
	~volumeChemData(void);

public:
	double getAmax(void);
	double getT(void);
	double getals(void);
	double getasl(void);
	double getDx(void);
	double getDy(void);
	double getsigx(void);
	double getKx(void);
	double getsigy(void);
	double getKy(void);
	double getk(void);
	double getm(void);
	double getexm(void);
	double getex(void);
	double getphi_eq(void);
	double gethnot(void);
	double getepsnot(void);
	double getmySign(void);
	double getQmax(void);
	double getSoc(void);
	double getEta(void);

private:
	double T;     // Temp in Kelvin
	double Amax;  // Maximum interfacial area 
	double Qmax;  // Maximum Charge/cm^3
	double soc;   // state of charge
	double eta;   // scale factor for SOC
	double als;   // transfer coefficient anodic
	double asl;   // transfer coefficient cathodic
	double Dx;    // diffusion coefficient in x-direction
	double Dy;    // diffusion coefficient in y-direction
	double sigx;   // conductivity of metal in Ohm/cm^2
	double Kx;     // conductivity o felectrolyte ohm/cm^2
	double sigy;   // conductivity of metal in Ohm/cm^2
	double Ky;     // conductivity o felectrolyte ohm/cm^2
	double k;     
	double m;
	double exm;
	double ex;
	double phi_eq;
	double hnot;
	double epsnot;
	double mySign;
};

#endif
